﻿/*************************************************************************
 * @name: fScroll
 * @author: BOSSMA
 * @version: 1.0
 * @date: 2014-04-01
 * @description: The fScroll is a javascript library, provides a simple way to implement horizontal/vertical scrolling of html elements.
 * @website: http://blog.bossma.cn
 * @copyright: You can use this program free,just need to keep the comment at the top.
 **************************************************************************/

/**
create a new fScroll instance.
 **/
function fScroll(options) {

	this.options = {
		containerId : options.containerId,
		direction : options.direction,
		interval : options.interval ? options.interval : 30,
		step : options.step ? options.step : 1,
		deceleration : options.deceleration ? options.deceleration : 1,
		useCustomPlace : options.useCustomPlace ? options.useCustomPlace : false,
		containerWidth : options.containerWidth,
		containerHeight : options.containerHeight,
		scrollContentWidth : options.scrollContentWidth,
		scrollContentHeight : options.scrollContentHeight
	}

	// internal variables
	this.scrollInterval = null;
	this.moveInterval = null;
	this.movedDistance = 0;
	this.isMoving = false;
	this.containerObject = document.getElementById(this.options.containerId);
	this.scrollItemWidth = 0;
	this.scrollItemHeight = 0;
	this.browser = {};
	this.GetBrowser();

	if (!this.containerObject) {
		return false;
	}

	// if not config the width of the container, get the width from css
	if (!this.options.containerWidth) {
		var cssWidth = this.GetCssValue(this.containerObject, 'width').replace("px", "");
		if (cssWidth) {
			if(isNaN(cssWidth)) {
				this.options.containerWidth = this.containerObject.offsetWidth;
			}else{
				this.options.containerWidth = cssWidth;
			}
		} else {
			this.options.containerWidth = this.containerObject.offsetWidth;
		}
	}

	//  if not config the height of the container, get the height from css
	if (!this.options.containerHeight) {
		var cssHeight = this.GetCssValue(this.containerObject, 'height').replace("px", "");
		if (cssHeight) {
			this.options.containerHeight = cssHeight;
		} else {
			this.options.containerHeight = this.containerObject.offsetHeight;
		}
	}

	// get the computed size of the container, and auto arrange elements in the container
	if (!this.options.useCustomPlace) {

		// get the container children
		var childArr = this.containerObject.children || this.containerObject.childNodes;

		var calculateWidth = 0;
		var calculateHeight = 0;

		if (this.options.direction == "left" || this.options.direction == "right") {
			for (i = 0; i < childArr.length; i++) {

				// arrange elements
				childArr[i].style.styleFloat = 'left';
				childArr[i].style.cssFloat = 'left';

				// solve the problem in IE
				if (this.browser.ie) {
					if (this.browser.ie == "5.5" || this.browser.ie == "6.0" || this.browser.ie == "7.0" || this.browser.ie == "8.0" || this.browser.ie == "9.0") {
						childArr[i].style.display = "inline";
						childArr[i].style.display = "inline";

						// no wrap
						childArr[i].style.whiteSpace = "nowrap";
					}
				}

				// get the width of the element occupied, and add to the total width
				var marginLeft = this.GetCssValue(childArr[i], 'marginLeft').replace("px", "");
				var marginRight = this.GetCssValue(childArr[i], 'marginRight').replace("px", "");
				var width = this.GetCssValue(childArr[i], 'width').replace("px", "");
				var borderLeftWidth = this.GetCssValue(childArr[i], 'borderLeftWidth').replace("px", "");
				var borderRightWidth = this.GetCssValue(childArr[i], 'borderRightWidth').replace("px", "");
				var paddingLeft = this.GetCssValue(childArr[i], 'paddingLeft').replace("px", "");
				var paddingRight = this.GetCssValue(childArr[i], 'paddingRight').replace("px", "");
				var offsetWidth = childArr[i].offsetWidth;

				var widthWithBorder = this.NumberParse(width) + this.NumberParse(borderLeftWidth) + this.NumberParse(borderRightWidth) + this.NumberParse(paddingLeft) + this.NumberParse(paddingRight);
				if (widthWithBorder > offsetWidth) {
					offsetWidth = widthWithBorder;
				}

				calculateWidth += this.NumberParse(offsetWidth) + this.NumberParse(marginLeft) + this.NumberParse(marginRight);

				// get the max height of the elements
				if (childArr[i].offsetHeight > calculateHeight) {
					calculateHeight = childArr[i].offsetHeight;
				}

				if (childArr[i].scrollHeight > this.scrollItemHeight) {
					this.scrollItemHeight = childArr[i].scrollHeight;
				}
			}

			//this.options.containerHeight = calculateHeight;
		}

		if (this.options.direction == "up" || this.options.direction == "down") {
			for (i = 0; i < childArr.length; i++) {
				// arrange elements
				childArr[i].style.clear = "both";

				// get the height of the element occupied, and add to the total height
				var marginTop = this.GetCssValue(childArr[i], 'marginTop').replace("px", "");
				var marginBottom = this.GetCssValue(childArr[i], 'marginBottom').replace("px", "");
				var height = this.GetCssValue(childArr[i], 'height').replace("px", "");
				var borderTopWidth = this.GetCssValue(childArr[i], 'borderTopWidth').replace("px", "");
				var borderBottomWidth = this.GetCssValue(childArr[i], 'borderBottomWidth').replace("px", "");
				var paddingTop = this.GetCssValue(childArr[i], 'paddingTop').replace("px", "");
				var paddingBottom = this.GetCssValue(childArr[i], 'paddingBottom').replace("px", "");
				var offsetHeight = childArr[i].offsetHeight;

				var heightWithBorder = this.NumberParse(height) + this.NumberParse(borderTopWidth) + this.NumberParse(borderBottomWidth) + this.NumberParse(paddingTop) + this.NumberParse(paddingBottom);
				if (heightWithBorder > offsetHeight) {
					offsetHeight = heightWithBorder;
				}

				calculateHeight += this.NumberParse(offsetHeight) + this.NumberParse(marginTop) + this.NumberParse(marginBottom);

				// get the max width of the elements
				if (childArr[i].offsetWidth > calculateWidth) {
					calculateWidth = childArr[i].offsetWidth;
				}

				if (childArr[i].scrollWidth > this.scrollItemWidth) {
					this.scrollItemWidth = childArr[i].scrollWidth;
				}
			}

			//this.options.containerWidth = calculateWidth;
		}
		
		this.options.scrollContentWidth = calculateWidth;
		this.options.scrollContentHeight = calculateHeight;
	}

	// set container style
	this.containerObject.style.overflow = "hidden";
	this.containerObject.style.width = this.options.containerWidth+"px";
	this.containerObject.style.height = this.options.containerHeight+"px";

	// get the content of the container
	var containerHtml = this.containerObject.innerHTML;
	
	if (this.options.direction == "left" || this.options.direction == "right") {
		if(this.options.containerWidth > this.options.scrollContentWidth){
			containerHtml+=this.containerObject.innerHTML;
			this.options.scrollContentWidth+=this.options.scrollContentWidth;
		}
	}

	// create two child elements with the same content for the container
	var scrollChild1 = document.createElement("div");
	scrollChild1.id = this.options.containerId + "_c1";
	scrollChild1.innerHTML = containerHtml;
	scrollChild1.style.width = this.options.scrollContentWidth+"px";

	var scrollChild2 = document.createElement("div");
	scrollChild2.id = this.options.containerId + "_c2";
	scrollChild2.innerHTML = containerHtml;
	scrollChild2.style.width = this.options.scrollContentWidth+"px";

	// clear the content of the container, and add the two child elements
	this.containerObject.innerHTML = "";
	this.containerObject.appendChild(scrollChild1);
	this.containerObject.appendChild(scrollChild2);

	// The internal elements width less than the actual width of container will wrap, need to increase the width
	if (!this.options.useCustomPlace) {
		if (this.options.direction == "left" || this.options.direction == "right") {
			while (scrollChild1.scrollHeight > this.scrollItemHeight && scrollChild1.scrollHeight > this.options.scrollContentHeight) {
				this.options.scrollContentWidth++;
				scrollChild1.style.width = this.options.scrollContentWidth+"px";
				scrollChild2.style.width = this.options.scrollContentWidth+"px";
			}
		}
	}

	scrollChild1.style.height = this.options.scrollContentHeight+"px";
	scrollChild2.style.height = this.options.scrollContentHeight+"px";

	// fix the scroll area position and layout
	if (this.options.direction == "down") {
		this.containerObject.scrollTop = this.options.scrollContentHeight * 2 - this.options.containerHeight;
	} else if (this.options.direction == "left" || this.options.direction == "right") {
		this.containerObject.style.whiteSpace = "nowrap";
		scrollChild1.style.display = "inline-block";
		scrollChild2.style.display = "inline-block";
		scrollChild1.style.zoom = "1";
		scrollChild2.style.zoom = "1";
		scrollChild1.style.margin = "0";
		scrollChild2.style.margin = "0";

		if (this.browser.ie) {
			if (this.browser.ie == "5.5" || this.browser.ie == "6.0" || this.browser.ie == "7.0") {
				scrollChild1.style.display = "inline";
				scrollChild2.style.display = "inline";
			}
		}

		if (this.options.direction == "right") {
			this.containerObject.scrollLeft = this.options.scrollContentWidth * 2 - this.options.containerWidth;
		}
	}

	var self = this;
	this.AttachEvent(this.containerObject, "mouseout", function () {
		self.Start()
	}, null);
	this.AttachEvent(this.containerObject, "mouseover", function () {
		self.Pause()
	}, null);
};

/**
pause scroll
 **/
fScroll.prototype.Pause = function () {
	var self = this;
	window.clearInterval(self.scrollInterval);
}

/**
start scroll
 **/
fScroll.prototype.Start = function () {

	if (!this.containerObject) {
		return false;
	}

	var self = this;
	this.scrollInterval = setInterval(function () {
			if (!self.isMoving) {
				self.Loop();
			}
		}, this.options.interval);
}

fScroll.prototype.Loop = function () {
	switch (this.options.direction) {
	case "up":
		if (this.options.deceleration > 1) {
			var obj = this;
			this.moveInterval = setInterval(function () {
					obj.SetScrollTop(obj.GetMoveSpeed());
				}, 30);
		} else {
			this.SetScrollTop(this.options.step);
		}
		break;
	case "down":
		if (this.options.deceleration > 1) {
			var obj = this;
			this.moveInterval = setInterval(function () {
					obj.SetScrollBottom(obj.GetMoveSpeed());
				}, 30);
		} else {
			this.SetScrollBottom(this.options.step);
		}
		break;
	case "left":
		if (this.options.deceleration > 1) {
			var obj = this;
			this.moveInterval = setInterval(function () {
					obj.SetScrollLeft(obj.GetMoveSpeed());
				}, 30);
		} else {
			this.SetScrollLeft(this.options.step);
		}
		break;
	case "right":
		if (this.options.deceleration > 1) {
			var obj = this;
			this.moveInterval = setInterval(function () {
					obj.SetScrollRight(obj.GetMoveSpeed());
				}, 30);
		} else {
			this.SetScrollRight(this.options.step);
		}
		break;
	}
}

/**
get the move distance within a specified time
 **/
fScroll.prototype.GetMoveSpeed = function () {
	var speed = 0;
	if (this.options.step - this.movedDistance > 0) {
		this.isMoving = true;
		speed = (this.options.step - this.movedDistance) / this.options.deceleration;
		speed = speed > 0 ? Math.ceil(speed) : Math.floor(speed);

		this.movedDistance = this.movedDistance + speed;
	} else {
		window.clearInterval(this.moveInterval);
		this.movedDistance = 0;
		this.isMoving = false;
	}

	return speed;
}

/**
set the scrollLeft of the container
 **/
fScroll.prototype.SetScrollLeft = function (speed) {
	if (speed > 0) {
		var scrollLeft = this.containerObject.scrollLeft;

		if (scrollLeft >= this.options.scrollContentWidth) {
			scrollLeft = speed;
		} else if (scrollLeft + speed > this.options.scrollContentWidth) {
			scrollLeft = scrollLeft + speed - this.options.scrollContentWidth;
		} else {
			scrollLeft = scrollLeft + speed;
		}

		this.containerObject.scrollLeft = scrollLeft;
	}
}

/**
set the scrollTop of the container
 **/
fScroll.prototype.SetScrollTop = function (speed) {
	if (speed > 0) {
		var scrollTop = this.containerObject.scrollTop;

		if (scrollTop >= this.options.scrollContentHeight) {
			scrollTop = speed;
		} else if (scrollTop + speed > this.options.scrollContentHeight) {
			scrollTop = scrollTop + speed - this.options.scrollContentHeight;
		} else {
			scrollTop = scrollTop + speed;
		}

		this.containerObject.scrollTop = scrollTop;
	}
}

/**
set the scrollRight of the container
 **/
fScroll.prototype.SetScrollRight = function (speed) {
	if (speed > 0) {
		var scrollLeft = this.containerObject.scrollLeft;

		if (scrollLeft <= 0) {
			scrollLeft = this.options.scrollContentWidth - speed;
		} else if (scrollLeft - speed < 0) {
			scrollLeft = this.options.scrollContentWidth + (scrollLeft - speed);
		} else {
			scrollLeft = scrollLeft - speed;
		}

		this.containerObject.scrollLeft = scrollLeft;
	}
}

/**
set the scrollBottom of the container
 **/
fScroll.prototype.SetScrollBottom = function (speed) {
	if (speed > 0) {
		var scrollTop = this.containerObject.scrollTop;

		if (scrollTop <= 0) {
			scrollTop = this.options.scrollContentHeight - speed;
		} else if (scrollTop - speed < 0) {
			scrollTop = this.options.scrollContentHeight + (scrollTop - speed);
		} else {
			scrollTop = scrollTop - speed;
		}

		this.containerObject.scrollTop = scrollTop;
	}
}

fScroll.prototype.AttachEvent = function (target, eventName, handler, argsObject) {
	var eventHandler = handler;
	if (argsObject) {
		eventHandler = function (e) {
			handler.call(argsObject, e);
		}
	}
	if (window.attachEvent) {
		target.attachEvent("on" + eventName, eventHandler);
	} else {
		target.addEventListener(eventName, eventHandler, false);
	}
}

fScroll.prototype.GetCssValue = function (element, cssName) {
	var view;
	if (element.ownerDocument && element.ownerDocument.defaultView) {
		view = element.ownerDocument.defaultView
	} else {
		view = element;
	}

	var cssValue = "";
	if (view.getComputedStyle) {
		cssValue = view.getComputedStyle(element, null)[cssName];
	} else {
		if (view.currentStyle) {
			cssValue = view.currentStyle[cssName];
		}
	}

	return cssValue;
}

fScroll.prototype.GetBrowser = function () {
	var ua = navigator.userAgent.toLowerCase();
	var s;
	(s = ua.match(/rv:([\d.]+)\) like gecko/)) ? this.browser.ie = s[1] :
		(s = ua.match(/msie ([\d.]+)/)) ? this.browser.ie = s[1] :
		(s = ua.match(/firefox\/([\d.]+)/)) ? this.browser.firefox = s[1] :
		(s = ua.match(/chrome\/([\d.]+)/)) ? this.browser.chrome = s[1] :
		(s = ua.match(/opera.([\d.]+)/)) ? this.browser.opera = s[1] :
		(s = ua.match(/version\/([\d.]+).*safari/)) ? this.browser.safari = s[1] : 0;
}

fScroll.prototype.NumberParse = function (str) {
	var result = Number(str);
	if(isNaN(result)){
		return 0;
	}
	return result;
}
